@@ -4,8 +4,7 @@ import struct
 import sys
 import traceback
 
-import avro
-import avro.io
+from fastavro import schemaless_reader, schemaless_writer
 
 from schema_registry.client.errors import ClientError
 from .errors import SerializerError, KeySerializerError, ValueSerializerError
@@ -14,14 +13,6 @@ log = logging.getLogger(__name__)
 
 MAGIC_BYTE = 0
 
-HAS_FAST = False
-try:
-    from fastavro import schemaless_reader, schemaless_writer
-
-    HAS_FAST = True
-except ImportError:
-    pass
-
 
 class ContextStringIO(io.BytesIO):
     """
@@ -54,14 +45,10 @@ class MessageSerializer:
         self.reader_value_schema = reader_value_schema
         self.schema_name_to_id = {}
 
-    def _get_encoder_func(self, writer_schema):
-        if HAS_FAST:
-            schema = writer_schema.to_json()
-            return lambda record, fp: schemaless_writer(fp, schema, record)
-        writer = avro.io.DatumWriter(writer_schema)
-        return lambda record, fp: writer.write(record, avro.io.BinaryEncoder(fp))
+    def _get_encoder_func(self, avro_schema):
+        return lambda record, fp: schemaless_writer(fp, avro_schema.schema, record)
 
-    def encode_record_with_schema(self, subject, schema, record, is_key=False):
+    def encode_record_with_schema(self, subject, avro_schema, record, is_key=False):
         """
         Given a parsed avro schema, encode a record for the given subject.
         The record is expected to be a dictionary.
@@ -76,7 +63,7 @@ class MessageSerializer:
         Returns:
             bytes: Encoded record with schema ID as bytes
         """
-        schema_id = self.schema_name_to_id.get(schema.name)
+        schema_id = self.schema_name_to_id.get(avro_schema.name)
 
         if not schema_id:
             serialize_err = KeySerializerError if is_key else ValueSerializerError
@@ -85,18 +72,18 @@ class MessageSerializer:
             # get the latest schema for the subject
             subject = f"{subject}{subject_suffix}"
             # register it
-            schema_id = self.schemaregistry_client.register(subject, schema)
+            schema_id = self.schemaregistry_client.register(subject, avro_schema)
 
             if not schema_id:
                 message = f"Unable to retrieve schema id for subject {subject}"
                 raise serialize_err(message)
 
             # cache writer
-            self.id_to_writers[schema_id] = self._get_encoder_func(schema)
+            self.id_to_writers[schema_id] = self._get_encoder_func(avro_schema)
 
             # cache the schema_id using the schema name
-            logging.info(f"Caching Schema {schema.name} with ID: {schema_id}")
-            self.schema_name_to_id[schema.name] = schema_id
+            logging.info(f"Caching Schema {avro_schema.name} with ID: {schema_id}")
+            self.schema_name_to_id[avro_schema.name] = schema_id
 
         return self.encode_record_with_schema_id(schema_id, record, is_key=is_key)
 
@@ -117,20 +104,17 @@ class MessageSerializer:
 
         # use slow avro
         if schema_id not in self.id_to_writers:
-            # get the writer + schema
-
             try:
-                schema = self.schemaregistry_client.get_by_id(schema_id)
-                if not schema:
+                avro_schema = self.schemaregistry_client.get_by_id(schema_id)
+                if not avro_schema:
                     raise serialize_err("Schema does not exist")
-                self.id_to_writers[schema_id] = self._get_encoder_func(schema)
+                self.id_to_writers[schema_id] = self._get_encoder_func(avro_schema)
             except ClientError:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 raise serialize_err(
                     repr(traceback.format_exception(exc_type, exc_value, exc_traceback))
                 )
 
-        # get the writer
         writer = self.id_to_writers[schema_id]
         with ContextStringIO() as outf:
             # Write the magic byte and schema ID in network byte order (big endian)
@@ -146,55 +130,19 @@ class MessageSerializer:
             return self.id_to_decoder_func[schema_id]
 
         try:
-            writer_schema_obj = self.schemaregistry_client.get_by_id(schema_id)
+            writer_schema = self.schemaregistry_client.get_by_id(schema_id)
         except ClientError as e:
             raise SerializerError(f"unable to fetch schema with id {schema_id}: {e}")
 
-        if writer_schema_obj is None:
+        if writer_schema is None:
             raise SerializerError(f"unable to fetch schema with id {schema_id}")
 
-        curr_pos = payload.tell()
+        reader_schema = self.reader_key_schema if is_key else self.reader_value_schema
 
-        reader_schema_obj = (
-            self.reader_key_schema if is_key else self.reader_value_schema
+        self.id_to_decoder_func[schema_id] = lambda payload: schemaless_reader(
+            payload, writer_schema.schema, reader_schema
         )
 
-        if HAS_FAST:
-            try:
-                writer_schema = writer_schema_obj.to_json()
-                reader_schema = reader_schema_obj.to_json()
-                schemaless_reader(payload, writer_schema)
-
-                # If we reach this point, this means we have fastavro and it can
-                # do this deserialization. Rewind since this method just determines
-                # the reader function and we need to deserialize again along the
-                # normal path.
-                payload.seek(curr_pos)
-
-                self.id_to_decoder_func[schema_id] = lambda p: schemaless_reader(
-                    p, writer_schema, reader_schema
-                )
-                return self.id_to_decoder_func[schema_id]
-            except Exception:
-                # Fast avro failed, fall thru to standard avro below.
-                pass
-
-        # here means we should just delegate to slow avro
-        # rewind
-        payload.seek(curr_pos)
-        # Avro DatumReader py2/py3 inconsistency, hence no param keywords
-        # should be revisited later
-        # https://github.com/apache/avro/blob/master/lang/py3/avro/io.py#L459
-        # https://github.com/apache/avro/blob/master/lang/py/src/avro/io.py#L423
-        # def __init__(self, writers_schema=None, readers_schema=None)
-        # def __init__(self, writer_schema=None, reader_schema=None)
-        avro_reader = avro.io.DatumReader(writer_schema_obj, reader_schema_obj)
-
-        def decoder(p):
-            bin_decoder = avro.io.BinaryDecoder(p)
-            return avro_reader.read(bin_decoder)
-
-        self.id_to_decoder_func[schema_id] = decoder
         return self.id_to_decoder_func[schema_id]
 
     def decode_message(self, message, is_key=False):
@@ -220,4 +168,5 @@ class MessageSerializer:
             if magic != MAGIC_BYTE:
                 raise SerializerError("message does not start with magic byte")
             decoder_func = self._get_decoder_func(schema_id, payload, is_key)
+
             return decoder_func(payload)
